@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2019 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2021 Live Networks, Inc.  All rights reserved.
 // 'Ogg' File Sink (recording a single media track only)
 // Implementation
 
@@ -78,7 +78,7 @@ OggFileSink::~OggFileSink() {
   OggFileSink::addData(fAltBuffer, fAltFrameSize, fAltPresentationTime);
 
   delete[] fAltBuffer;
-  delete[] fConfigStr;
+  delete[] (char*)fConfigStr;
 }
 
 Boolean OggFileSink::continuePlaying() {
