@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2020 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2021 Live Networks, Inc.  All rights reserved.
 // RTCP
 // Implementation
 
@@ -117,7 +117,7 @@ static double dTimeNow() {
 }
 
 static unsigned const maxRTCPPacketSize = 1438;
-	// bytes (1500, minus some allowance for IP, UDP, UMTP headers; SRTCP trailers)
+	// bytes (1500, minus some allowance for IP, UDP headers; SRTCP trailers)
 static unsigned const preferredRTCPPacketSize = 1000; // bytes
 
 RTCPInstance::RTCPInstance(UsageEnvironment& env, Groupsock* RTCPgs,
@@ -217,26 +217,22 @@ void RTCPInstance::noteArrivingRR(struct sockaddr_storage const& fromAddressAndP
 
   // Specific RR handler:
   if (fSpecificRRHandlerTable != NULL) {
-    netAddressBits fromAddr;
+    struct sockaddr_storage fromAddress;
     portNumBits fromPortNum;
     if (tcpSocketNum < 0) {
       // Normal case: We read the RTCP packet over UDP
-      if (fromAddressAndPort.ss_family == AF_INET) {
-	struct sockaddr_in const& fromAddressAndPort4 = (struct sockaddr_in const&)fromAddressAndPort;
-	fromAddr = fromAddressAndPort4.sin_addr.s_addr;
-      } else {
-	fromAddr = 0; // later update for IPv6
-      }
+      fromAddress = fromAddressAndPort;
       fromPortNum = ntohs(portNum(fromAddressAndPort));
     } else {
       // Special case: We read the RTCP packet over TCP (interleaved)
-      // Hack: Use the TCP socket and channel id to look up the handler
-      fromAddr = tcpSocketNum;
+      // Hack: Use the TCP socket number and channel id to look up the handler
+      fromAddress.ss_family = AF_INET;
+      ((sockaddr_in&)fromAddress).sin_addr.s_addr = tcpSocketNum;
       fromPortNum = tcpStreamChannelId;
     }
     Port fromPort(fromPortNum);
     RRHandlerRecord* rrHandler
-      = (RRHandlerRecord*)(fSpecificRRHandlerTable->Lookup(fromAddr, (~0), fromPort));
+      = (RRHandlerRecord*)(fSpecificRRHandlerTable->Lookup(fromAddress, fromPort));
     if (rrHandler != NULL) {
       if (rrHandler->rrHandlerTask != NULL) {
 	(*(rrHandler->rrHandlerTask))(rrHandler->rrHandlerClientData);
@@ -312,7 +308,7 @@ void RTCPInstance::setRRHandler(TaskFunc* handlerTask, void* clientData) {
 }
 
 void RTCPInstance
-::setSpecificRRHandler(netAddressBits fromAddress, Port fromPort,
+::setSpecificRRHandler(struct sockaddr_storage const& fromAddress, Port fromPort,
 		       TaskFunc* handlerTask, void* clientData) {
   if (handlerTask == NULL && clientData == NULL) {
     unsetSpecificRRHandler(fromAddress, fromPort);
@@ -325,19 +321,20 @@ void RTCPInstance
   if (fSpecificRRHandlerTable == NULL) {
     fSpecificRRHandlerTable = new AddressPortLookupTable;
   }
-  RRHandlerRecord* existingRecord = (RRHandlerRecord*)fSpecificRRHandlerTable->Add(fromAddress, (~0), fromPort, rrHandler);
+  RRHandlerRecord* existingRecord
+    = (RRHandlerRecord*)fSpecificRRHandlerTable->Add(fromAddress, fromPort, rrHandler);
   delete existingRecord; // if any
 
 }
 
 void RTCPInstance
-::unsetSpecificRRHandler(netAddressBits fromAddress, Port fromPort) {
+::unsetSpecificRRHandler(struct sockaddr_storage const& fromAddress, Port fromPort) {
   if (fSpecificRRHandlerTable == NULL) return;
 
   RRHandlerRecord* rrHandler
-    = (RRHandlerRecord*)(fSpecificRRHandlerTable->Lookup(fromAddress, (~0), fromPort));
+    = (RRHandlerRecord*)(fSpecificRRHandlerTable->Lookup(fromAddress, fromPort));
   if (rrHandler != NULL) {
-    fSpecificRRHandlerTable->Remove(fromAddress, (~0), fromPort);
+    fSpecificRRHandlerTable->Remove(fromAddress, fromPort);
     delete rrHandler;
   }
 }
@@ -382,13 +379,13 @@ void RTCPInstance::sendAppPacket(u_int8_t subtype, char const* name,
   sendBuiltPacket();
 }
 
-void RTCPInstance::setStreamSocket(int sockNum,
-				   unsigned char streamChannelId) {
+void RTCPInstance::setStreamSocket(int sockNum, unsigned char streamChannelId,
+				   TLSState* tlsState) {
   // Turn off background read handling:
   fRTCPInterface.stopNetworkReading();
 
   // Switch to RTCP-over-TCP:
-  fRTCPInterface.setStreamSocket(sockNum, streamChannelId);
+  fRTCPInterface.setStreamSocket(sockNum, streamChannelId, tlsState);
 
   // Turn background reading back on:
   TaskScheduler::BackgroundHandlerProc* handler
@@ -396,13 +393,13 @@ void RTCPInstance::setStreamSocket(int sockNum,
   fRTCPInterface.startNetworkReading(handler);
 }
 
-void RTCPInstance::addStreamSocket(int sockNum,
-				   unsigned char streamChannelId) {
+void RTCPInstance::addStreamSocket(int sockNum, unsigned char streamChannelId,
+				   TLSState* tlsState) {
   // First, turn off background read handling for the default (UDP) socket:
   envir().taskScheduler().turnOffBackgroundReadHandling(fRTCPInterface.gs()->socketNum());
 
   // Add the RTCP-over-TCP interface:
-  fRTCPInterface.addStreamSocket(sockNum, streamChannelId);
+  fRTCPInterface.addStreamSocket(sockNum, streamChannelId, tlsState);
 
   // Turn on background reading for this socket (in case it's not on already):
   TaskScheduler::BackgroundHandlerProc* handler
@@ -520,7 +517,7 @@ void RTCPInstance
     fprintf(stderr, "[%p]saw incoming RTCP packet (from ", this);
     if (tcpSocketNum < 0) {
       // Note that "fromAddressAndPort" is valid only if we're receiving over UDP (not over TCP):
-      fprintf(stderr, "address %s, port %d", AddressString(fromAddressAndPort).val(), ntohs(portNum(fromAddressAndPort));
+      fprintf(stderr, "address %s, port %d", AddressString(fromAddressAndPort).val(), ntohs(portNum(fromAddressAndPort)));
     } else {
       fprintf(stderr, "TCP socket #%d, stream channel id %d", tcpSocketNum, tcpStreamChannelId);
     }
