@@ -13,14 +13,14 @@ You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
-// Copyright (c) 1996-2020, Live Networks, Inc.  All rights reserved
+// Copyright (c) 1996-2021, Live Networks, Inc.  All rights reserved
 // A test program that streams a MP3 file via RTP/RTCP
 // main program
 
 #include "liveMedia.hh"
-#include "GroupsockHelper.hh"
 
 #include "BasicUsageEnvironment.hh"
+#include "announceURL.hh"
 
 // To stream using 'ADUs' rather than raw MP3 frames, uncomment the following:
 //#define STREAM_USING_ADUS 1
@@ -31,16 +31,20 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 
 // To stream using "source-specific multicast" (SSM), uncomment the following:
 //#define USE_SSM 1
+
+// To stream using IPv6 multicast, rather than IPv4 multicast, uncomment the following:
+//#define USE_IPV6_MULTICAST 1
+
+// To set up an internal RTSP server, uncomment the following:
+//#define IMPLEMENT_RTSP_SERVER 1
+// (Note that this RTSP server works for multicast only)
+
 #ifdef USE_SSM
 Boolean const isSSM = True;
 #else
 Boolean const isSSM = False;
 #endif
 
-// To set up an internal RTSP server, uncomment the following:
-//#define IMPLEMENT_RTSP_SERVER 1
-// (Note that this RTSP server works for multicast only)
-
 #ifdef IMPLEMENT_RTSP_SERVER
 RTSPServer* rtspServer;
 #endif
@@ -68,21 +72,32 @@ int main(int argc, char** argv) {
 
   // Create 'groupsocks' for RTP and RTCP:
   char const* destinationAddressStr
+#ifdef USE_IPV6_MULTICAST
+#ifdef USE_SSM
+    = "FF3E::FFFF:2A2A";
+#else
+    = "FF1E::FFFF:2A2A";
+#endif
+#else  
 #ifdef USE_SSM
     = "232.255.42.42";
 #else
     = "239.255.42.42";
+#endif
+#endif
   // Note: This is a multicast address.  If you wish to stream using
   // unicast instead, then replace this string with the unicast address
   // of the (single) destination.  (You may also need to make a similar
   // change to the receiver program.)
-#endif
+
   const unsigned short rtpPortNum = 6666;
   const unsigned short rtcpPortNum = rtpPortNum+1;
   const unsigned char ttl = 1; // low, in case routers don't admin scope
 
-  struct in_addr destinationAddress;
-  (void)inet_pton(AF_INET, destinationAddressStr, &destinationAddress.s_addr);
+  NetAddressList destinationAddresses(destinationAddressStr);
+  struct sockaddr_storage destinationAddress;
+  copyAddress(destinationAddress, destinationAddresses.firstAddress());
+
   const Port rtpPort(rtpPortNum);
   const Port rtcpPort(rtcpPortNum);
 
@@ -120,10 +135,7 @@ int main(int argc, char** argv) {
   // Note: This starts RTCP running automatically
 
 #ifdef IMPLEMENT_RTSP_SERVER
-  rtspServer = RTSPServer::createNew(*env);
-  // Note that this (attempts to) start a server on the default RTSP server
-  // port: 554.  To use a different port number, add it as an extra
-  // (optional) parameter to the "RTSPServer::createNew()" call above.
+  rtspServer = RTSPServer::createNew(*env, 8554);
   if (rtspServer == NULL) {
     *env << "Failed to create RTSP server: " << env->getResultMsg() << "\n";
     exit(1);
@@ -133,10 +145,7 @@ int main(int argc, char** argv) {
 		"Session streamed by \"testMP3Streamer\"", isSSM);
   sms->addSubsession(PassiveServerMediaSubsession::createNew(*sessionState.sink, sessionState.rtcpInstance));
   rtspServer->addServerMediaSession(sms);
-
-  char* url = rtspServer->rtspURL(sms);
-  *env << "Play this stream using the URL \"" << url << "\"\n";
-  delete[] url;
+  announceURL(rtspServer, sms);
 #endif
 
   play();
